local Addon_Name, bc = ... 

local rollers = {}
RaidRollsShown = true
local FSPool = {}
local _wasInGroup = nil
local PASS = "PASS"
local CHAT_MSG_EVENTS = {
    "CHAT_MSG_BATTLEGROUND",
    "CHAT_MSG_BATTLEGROUND_LEADER",
    "CHAT_MSG_PARTY_LEADER",
    "CHAT_MSG_PARTY",
    "CHAT_MSG_RAID_LEADER",
    "CHAT_MSG_RAID",
    "CHAT_MSG_WHISPER",
    --~ "CHAT_MSG_BN_WHISPER",
    }


local function getFS(i)
    local f = FSPool[i]
    if not f then
        f = {}
        l = RaidRolls_MainFrame:CreateFontString("$parent_FSL" .. tostring(i), "RaidRolls_MainFrame", "GameTooltipText")
        l:SetHeight(20)
        r = RaidRolls_MainFrame:CreateFontString("$parent_FSR" .. tostring(i), "RaidRolls_MainFrame", "GameTooltipText")
        r:SetHeight(20)
        
        f = { L = l, R = r}
        tinsert(FSPool, f)
    else
        f.L:Show()
        f.R:Show()
    end
    return f
end


function RaidRolls_ParseText(chatEntry, send)
     -- This function actually gets called every time the user hits a key. But the
     -- send flag will only be set when he hits return to send the message.
     if (send == 1) then
        -- Player "pass" only in group.
        if (RaidRolls_groupType() == nil) then return end
        
        local msg = chatEntry:GetText(); -- Here's how you get the original text
        if (string.lower(msg) == "pass") then
            local playerName = UnitName("player");
            rollers[playerName] = PASS
            RaidRolls_update()
        end
     end
end


function onload(self)
    print(GetAddOnMetadata("RaidRolls", "Title") .. " v" .. GetAddOnMetadata("RaidRolls", "Version") .. " loaded. Type /raidrolls for help.");
end


local function name2raidIndex(target)
    for i = 1, GetNumRaidMembers() do
        local name = GetRaidRosterInfo(i)
        if (name == target) then
            return i
        end
    end
    
    return nil
end


local function myLength(tab)
    local count = 0
    for k,v in pairs(tab) do
        count = count + 1
    end
    return count
end


function RaidRolls_groupType()
    local _groupType = nil
    
    if (GetNumRaidMembers() > 0) then
        _groupType = "RAID"
    elseif (GetNumPartyMembers() > 0) then
        _groupType = "PARTY"
    end
    
    return _groupType
end


--- Was group joined/leaved since last call?
-- @return Group: joined (true), leaved(false), neither (nil).
local function groupTypeChanged()
    local outcome = nil
    local _groupType = RaidRolls_groupType()
    
    if (_wasInGroup ~= nil) then  -- Not init.
        if (_groupType == nil) then  -- No group.
            if (_wasInGroup) then
                outcome = true
            else
                outcome = false
            end
        else  -- Group.
            if (_wasInGroup) then
                outcome = false
            else
                outcome = true
            end
        end
    end
    
    -- New value.
    if (_groupType == nil) then
        _wasInGroup = false
    else
        _wasInGroup = true
    end
    return outcome
end


-- Any param (other than nil) make the function work even out of group.
-- Used for rolls reset.
function RaidRolls_update(param)
    local _groupType = RaidRolls_groupType()
    if (_groupType == nil and param == nil) then return end
    
    RaidRolls_MainFrame:SetHeight(5 + 15 + 10 + 20 * myLength(rollers));
    
    local sortedRollers = {}
    for k, v in pairs(rollers) do
        -- k = name; v = roll
        table.insert(sortedRollers, { k = k, v = v })
    end
    table.sort(sortedRollers,
        function(lhs, rhs)
            if (lhs.v == PASS or rhs.v == PASS) then return false end
            return math.abs(lhs.v) > math.abs(rhs.v)
        end)
    
    local i = 1
    for k, v in pairs(sortedRollers) do
        -- v.k = name; v.v = roll
        
        local Name, subgroup, class, fileName
        if (_groupType == "RAID") then
            local index = name2raidIndex(v.k)
            if (index == nil) then return end
            Name, _, subgroup, _, class, fileName = GetRaidRosterInfo(index)
        elseif (_groupType == "PARTY") then
            if (not UnitInParty(v.k)) then return end
            Name = v.k
            subgroup = "P"
            class, fileName = UnitClass(v.k)
        end
        
        local roll = ""
        if (v.v == PASS) then
            roll = "|cFF00ccffpass|r"
        elseif (v.v > 0) then
            roll = v.v
        else
            roll = "|cFFFF0000" .. math.abs(v.v) .. "|r"
        end
        
        local parent = ""
        if (i == 1) then
            parentL = "$parent_Player"
            parentR = "$parent_Roll"
        else
            parentL = "$parent_FSL" .. tostring(i - 1)
            parentR = "$parent_FSR" .. tostring(i - 1)
        end
        local FS = getFS(i)
        FS.L:SetText(Name .. " (" .. bc.classColours[fileName] .. class .. "|r)[" .. bc.channelColours[_groupType] .. subgroup .. "|r]")
        FS.R:SetText(roll)
        FS.L:SetPoint("TOPLEFT", parentL, "BOTTOMLEFT")
        FS.R:SetPoint("TOPLEFT", parentR, "BOTTOMLEFT")
        
        i = i + 1
    end
    
    local myLengthFSPool = myLength(FSPool)
    while (i <= myLengthFSPool) do
        FS = getFS(i)
        FS.L:Hide()
        FS.R:Hide()
        i = i + 1
    end
end


local function reset()
    rollers = {}
    RaidRolls_update("X")
end
bc.reset = reset


local function show(bool)
    RaidRollsShown = bool
    if (bool) then
        RaidRolls_MainFrame:Show()
    else
        RaidRolls_MainFrame:Hide()
    end
end
bc.show = show


local ChatSystem_EventFrame = CreateFrame("Frame")
ChatSystem_EventFrame:RegisterEvent("CHAT_MSG_SYSTEM")
ChatSystem_EventFrame:SetScript("OnEvent",
    function(self, event, ...)
        if (RaidRolls_groupType() == nil) then return end
        
        local msg = select(1, ...)
        if msg then
            
            -- Roll message.
            if (string.find(msg, "rolls") ~= nil) then
                local name, roll, minRoll, maxRoll = msg:match("^(.+) rolls (%d+) %((%d+)%-(%d+)%)$")
                local minRoll = tonumber(minRoll)
                local maxRoll = tonumber(maxRoll)

                if not(name and minRoll == 1 and maxRoll == 100) then return end
                  
                if (rollers[name] == nil) then
                    rollers[name] = tonumber(roll)
                else
                    --~ rollers[name] = -math.max(math.abs(rollers[name]), roll)
                    rollers[name] = -roll
                end
                
                RaidRolls_update()
            
            -- Leave raid msg.
            elseif (string.find(msg, "has left the raid group.") ~= nil) then
                local name = msg:match("^(.+) has left the raid group.$")
                if name then
                    rollers[name] = nil
                    RaidRolls_update()
                end
            
            -- Leave party msg.
            elseif (string.find(msg, "leaves the party.") ~= nil) then
                local name = msg:match("^(.+) leaves the party.$")
                if name then
                    rollers[name] = nil
                    RaidRolls_update()
                end
            end
        end
    end)


local Raid_EventFrame = CreateFrame("Frame")
Raid_EventFrame:RegisterEvent("PARTY_MEMBERS_CHANGED")
Raid_EventFrame:SetScript("OnEvent",
    function(self, event, ...)
        
        if (groupTypeChanged()) then  -- Join or leave party/raid.
            if (RaidRolls_groupType() == nil) then  -- Just left.
                RairRolls_ChatGroup_EventFrame_UnregisterEvents()
            else  -- Just joined.
                RairRolls_ChatGroup_EventFrame_RegisterEvents()
            end
        end
        
        RaidRolls_update()
        
    end)


local ChatGroup_EventFrame = CreateFrame("Frame")
ChatGroup_EventFrame:SetScript("OnEvent",
    function(self, event, ...)
        
        if (RaidRolls_groupType() == nil) then return end
        
        local msg, name = ...
        if (string.lower(msg) == "pass") then
            rollers[name] = PASS
            RaidRolls_update()
        end
    end)


function RairRolls_ChatGroup_EventFrame_RegisterEvents()
    for i, event in ipairs(CHAT_MSG_EVENTS) do
        ChatGroup_EventFrame:RegisterEvent(event)
    end
end


function RairRolls_ChatGroup_EventFrame_UnregisterEvents()
    for i, event in ipairs(CHAT_MSG_EVENTS) do
        ChatGroup_EventFrame:UnregisterEvent(event)
    end
end


local Load_EventFrame = CreateFrame("Frame")
Load_EventFrame:RegisterEvent("ADDON_LOADED")
Load_EventFrame:SetScript("OnEvent",
    function(self, event, ...)
        if (event == "ADDON_LOADED" and ... == "RaidRolls") then
            
            if RaidRollsShown == nil then
                -- This is the first time this addon
                -- is loaded; initialize to true.
                RaidRollsShown = true;
            end
            show(RaidRollsShown)
            
            RairRolls_ChatGroup_EventFrame_RegisterEvents()
            
            hooksecurefunc('ChatEdit_ParseText', RaidRolls_ParseText);
            
            groupTypeChanged()
        end
    end)
